#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/time.h>
#include <sys/mman.h>
#include <pthread.h>
#include <time.h>
#include <signal.h>
#include <semaphore.h>

#include "csapi.h"
#include "csi2c.h"
#include "cshdmirx.h"

typedef enum
{
    UNKNOW,
    PLAY = 0x11,
    STOP = 0x12,
    OPENED = 0x13,
    CLOSED = 0x14
} PLAY_STATUS;

typedef struct _VideoOpt_t
{

    sem_t              lock;
    pthread_t          task;
    pthread_t          read_enc_task;

    FILE               *input_fp;
    FILE               *output_fp;
    char               *input_file;
    char               *output_file;

    int                loop;
    int                restart;
    int                phase;
    int                dev_index;
    int                width;
    int                height;
    //CSVID_Crop         crop;
    //CSVID_Rect         rect;
    CSVID_DEV_TYPE     device_type;
    CSVID_STREAM_TYPE  input_codec;
    CSVID_STREAM_TYPE  output_codec;
    CSTVOUT_MODE       tv_mode;
    PLAY_STATUS        play_status;
    CSTVOUT_HANDLE     tvout_handle;
    CSVID_HANDLE       dec_handle;
    CSVID_HANDLE       enc_handle;
    CSVID_HANDLE       cap_handle;
    //CSVID_HANDLE       transcoding_handle;
} VideoOpt_t;

static void show_video_help(void);
static void show_command(void);
static char get_command(void);
static int process_command(VideoOpt_t *pVideoOpt, char cmd);
static void* transcoding_proc(void *params);
static void *write_stream_to_decoder(void *params);
static void* write_stream_to_decoder_by_frame(void *params); 
static void *write_yuv_to_display(void *params);
static void *read_stream_from_encoder(void *params);
static void *do_from_encoder_to_decoder(void *params);
static void *read_yuv_from_decoder(void *params);
static int parse_video_cmdline(int argc, char **argv, VideoOpt_t *pVideoOpt);

static void GetRxTvMode(unsigned int *srcWidth, unsigned int *srcHeight, unsigned int *is_interlace);
static void GetOutputTvMOde(CSTVOUT_MODE cur_mode, unsigned int *dstWidth, unsigned int *dstHeight, unsigned int *is_interlace);

#ifndef RAMDOM_BLOCK_SIZE
#define RAMDOM_BLOCK_SIZE
#endif

#ifdef RAMDOM_BLOCK_SIZE
#define random(x) (rand()%x)
#define READFILE_SIZE (1*1024*1024)
#else
#define random(x) (x)
#define READFILE_SIZE (4096)
#endif
#define VID_DEV_TYPE_DECODER_SLOW 100

static VideoOpt_t pVideoOpt;
static unsigned int input_interlace = 0, output_interlace = 0;
static unsigned int dstHeight=0, dstWidth=0, srcWidth = 0, srcHeight = 0;

#define TEMP_BUF_LEN  (2048*1088*2)
static unsigned char YUVTempBuffer[TEMP_BUF_LEN];
static unsigned char YUVTempBuffer1[TEMP_BUF_LEN];

static void abort_quit(int sign)
{
    switch(sign)
    {
    case SIGINT:
        if(pVideoOpt.play_status == CLOSED)
        {
            CSVID_Term();
            exit(-1);
        }
        if(pVideoOpt.device_type == VID_DEV_TYPE_DECODER)
        {
            if(pVideoOpt.play_status == PLAY)
            {
                sem_wait(&pVideoOpt.lock);
                CSVID_Stop(pVideoOpt.dec_handle);
            }
            sem_destroy(&pVideoOpt.lock);
            if(pVideoOpt.input_fp)
            {
                fclose(pVideoOpt.input_fp);
                pVideoOpt.input_fp = NULL;
            }
            CSVID_SyncNotify(pVideoOpt.dec_handle, NULL, 1000000, 0);
            CSVID_Close(pVideoOpt.dec_handle);
        }
        else if(pVideoOpt.device_type == VID_DEV_TYPE_DECODER_SLOW)
        {
            if(pVideoOpt.play_status == PLAY)
            {
                sem_wait(&pVideoOpt.lock);
                CSVID_Stop(pVideoOpt.dec_handle);
            }
            sem_destroy(&pVideoOpt.lock);
            if(pVideoOpt.input_fp)
            {
                fclose(pVideoOpt.input_fp);
                pVideoOpt.input_fp = NULL;
            }
            CSVID_SyncNotify(pVideoOpt.dec_handle, NULL, 1000000, 0);
            CSVID_Close(pVideoOpt.dec_handle);
        }
        else if(pVideoOpt.device_type == VID_DEV_TYPE_ENCODER)
        {
            if(pVideoOpt.play_status == PLAY)
            {
                sem_wait(&pVideoOpt.lock);
                CSVID_StopEncoder(pVideoOpt.enc_handle);
            }
            sem_destroy(&pVideoOpt.lock);
            if(pVideoOpt.input_fp)
            {
                fclose(pVideoOpt.input_fp);
                pVideoOpt.input_fp = NULL;
            }
            CSVID_CloseEncoder(pVideoOpt.enc_handle);
        }
        else if(pVideoOpt.device_type == VID_DEV_TYPE_CAPTURE)
        {
            if(pVideoOpt.play_status == PLAY)
            {
                CSVID_StopCapture(pVideoOpt.cap_handle);
            }
            CSVID_CloseCapture(pVideoOpt.cap_handle);

        }
        else if(pVideoOpt.device_type == VID_DEV_TYPE_MULTI_CHANNEL)
        {
            if(pVideoOpt.play_status == PLAY)
            {
                sem_wait(&pVideoOpt.lock);
                CSVID_StopEncoder(pVideoOpt.enc_handle);
                CSVID_Stop(pVideoOpt.dec_handle);
                pVideoOpt.play_status = STOP;
            }
            sem_destroy(&pVideoOpt.lock);
            CSVID_Close(pVideoOpt.dec_handle);
            CSVID_CloseEncoder(pVideoOpt.enc_handle);
            CSVID_CloseCapture(pVideoOpt.cap_handle);
        }
        else if(pVideoOpt.device_type == VID_DEV_TYPE_TRANSCODING)
        {
            if(pVideoOpt.play_status == PLAY)
            {
                sem_wait(&pVideoOpt.lock);
                CSVID_StopTranscoding(pVideoOpt.dec_handle, pVideoOpt.enc_handle);
                pVideoOpt.play_status = STOP;
            }
            sem_destroy(&pVideoOpt.lock);
            CSVID_CloseTranscoding(pVideoOpt.dec_handle, pVideoOpt.enc_handle);
        }

        CSVID_Term();
        exit(-1);
        break;
    default:
        ;
    }
}


void synccall(CSVID_HANDLE handle, CSVID_SYNC *is_sync)
{
    UNUSED_VARIABLE(handle);
    switch(*is_sync)
    {
    case VID_SYNC_PICTURE_READY:
        //printf("%s VID_SYNC_PICTURE_READY\n", __FUNCTION__);
        break;
    case VID_SYNC_AV_SYNC:
        //printf("%s VID_SYNC_AV_SYNC\n", __FUNCTION__);
        break;
    case VID_SYNC_TIMEOUT:
        //printf("%s VID_SYNC_TIMEOUT\n", __FUNCTION__);
        break;
    default:
        printf(" no sync handle\n");
    }
}
void src_format_change(CSVID_HANDLE handle, CSVID_SequenceHeader *hdr)
{
    UNUSED_VARIABLE(handle);
    UNUSED_VARIABLE(hdr);
}
void __csvid_printf_status(CSVID_DEV dev);
int main(int argc, char *argv[])
{
    EncCpbAttr EncAttr;
    CSVID_Rect SrcRect, DstRect;
    CSVID_Init_Params params;
    params.local_framerate = VID_FRAME_RATE_3000;

    if(argc < 3)
    {
        show_video_help();
        exit(-1);
    }
    memset(&pVideoOpt, 0, sizeof(VideoOpt_t));

    pVideoOpt.loop = 1;
    pVideoOpt.device_type = VID_DEV_TYPE_TRANSCODING;
    pVideoOpt.dev_index = 0;
    pVideoOpt.tv_mode = TVOUT_MODE_1080P60;
    pVideoOpt.input_codec = VID_STREAM_TYPE_H264;
    pVideoOpt.output_codec = VID_STREAM_TYPE_H264;
    pVideoOpt.width = 1920;
    pVideoOpt.height = 1080;
    //pVideoOpt.crop.CropXOff = 0x0;
    //pVideoOpt.crop.CropXWidth = 2000;
    //pVideoOpt.crop.CropYOff = 0x0;
    //pVideoOpt.crop.CropYHeight = 2000;

    if(0 != parse_video_cmdline(argc, argv, &pVideoOpt))
    {
        printf("Incorrect paramarters\n\n");
        show_video_help();
        return -1;
    }

    pVideoOpt.tvout_handle = CSTVOUT_Open(TVOUT_CHANNEL0);
    if(NULL == pVideoOpt.tvout_handle)
    {
        printf("Open tvout channel 0 failed\n");
        goto fail_open_tvout;;
    }

    if(CSAPI_FAILED == CSVID_Init(params))
    {
        printf("Initialing video firmware failed\n");
        goto fail_init_videofw;
    }

    CSTVOUT_SetMode(pVideoOpt.tvout_handle, pVideoOpt.tv_mode);

    sem_init(&pVideoOpt.lock, 0, 0);

    if(pVideoOpt.device_type == VID_DEV_TYPE_DECODER)
    {

        if(NULL == (pVideoOpt.dec_handle = CSVID_Open(pVideoOpt.dev_index)))
        {
            printf("Open deooder %d failed\n", pVideoOpt.dev_index);
            goto fail_open_dev;
        }

        CSVID_SRCFormatChangeNotify(pVideoOpt.dec_handle,src_format_change, 1);

//CSAPI_RESULT CSVID_SRCFormatChangeNotify(CSVID_HANDLE handle, void (*call_back_function) (CSVID_HANDLE,CSVID_SequenceHeader *), int event_enable)
        //CSVID_Set_Output_Layer(pVideoOpt.dec_handle, VIDEO_LAYER_0);
        //CSVID_SetLayerPostion(pVideoOpt.dec_handle, pVideoOpt.rect);
        //CSVID_SetLayerOrder(pVideoOpt.dec_handle, 0);
        //CSVID_SetOutputPostion(pVideoOpt.dec_handle, pVideoOpt.crop, pVideoOpt.rect);
        //CSVID_SetDecoderMode(pVideoOpt.dec_handle, VID_FRAME_ANY);

        GetOutputTvMOde(pVideoOpt.tv_mode,&dstWidth, &dstHeight, &output_interlace);

        DstRect.left = 0;
        DstRect.right = dstWidth;
        DstRect.top   = 0;
        DstRect.bottom = dstHeight;

        CSVID_SetStreamType(pVideoOpt.dec_handle, pVideoOpt.input_codec);
        CSVID_SetDecoderReorder(pVideoOpt.dec_handle, 1);
        CSVID_EnablePTSSync(pVideoOpt.dec_handle);
        CSVID_SetVideoWithAudio(pVideoOpt.dec_handle, 0);
        CSVID_SyncNotify(pVideoOpt.dec_handle, synccall, 1000000, 1);

        SrcRect.left = 0;
        SrcRect.right = pVideoOpt.width;
        SrcRect.top   = 0;
        SrcRect.bottom = pVideoOpt.height;
        CSVID_SetSrcPosition(pVideoOpt.dec_handle, pVideoOpt.width, pVideoOpt.height, &SrcRect, 0);
        CSVID_SetDstPosition(pVideoOpt.dec_handle, dstWidth, dstHeight, &DstRect, output_interlace);

        pVideoOpt.play_status = OPENED;
    }
    else if(pVideoOpt.device_type == VID_DEV_TYPE_DECODER_SLOW)
    {
        if(NULL == (pVideoOpt.dec_handle = CSVID_Open(pVideoOpt.dev_index)))
        {
            printf("Open deooder %d failed\n", pVideoOpt.dev_index);
            goto fail_open_dev;
        }

        CSVID_SRCFormatChangeNotify(pVideoOpt.dec_handle,src_format_change, 1);

        CSVID_SetDecoderMode(pVideoOpt.dec_handle, VID_FRAME_ANY);

        GetOutputTvMOde(pVideoOpt.tv_mode,&dstWidth, &dstHeight, &output_interlace);

        DstRect.left = 0;
        DstRect.right = dstWidth;
        DstRect.top   = 0;
        DstRect.bottom = dstHeight;

        CSVID_SetStreamType(pVideoOpt.dec_handle, pVideoOpt.input_codec);
        CSVID_SetDecoderReorder(pVideoOpt.dec_handle, 0);
        CSVID_DisablePTSSync(pVideoOpt.dec_handle);
        //CSVID_EnablePTSSync(pVideoOpt.dec_handle);
        //CSVID_SyncNotify(pVideoOpt.dec_handle, synccall, 1000000, 1);
        CSVID_SetVideoWithAudio(pVideoOpt.dec_handle, 0);
        CSVID_SetLowLatencyMode(pVideoOpt.dec_handle, VID_LLD_ALL, 10);

        SrcRect.left = 0;
        SrcRect.right = pVideoOpt.width;
        SrcRect.top   = 0;
        SrcRect.bottom = pVideoOpt.height;
        CSVID_SetSrcPosition(pVideoOpt.dec_handle, pVideoOpt.width, pVideoOpt.height, &SrcRect, 0);
        CSVID_SetDstPosition(pVideoOpt.dec_handle, dstWidth, dstHeight, &DstRect, output_interlace);

        pVideoOpt.play_status = OPENED;

    }
    else if(pVideoOpt.device_type == VID_DEV_TYPE_ENCODER)
    {
        EncSliceMode sliceMode;

        CSHDMIRX_Open();
        CSHDMIRX_SetRepeaterMode(0);
        CSHDMIRX_SetHpd(1);
        CSHDMIRX_SetManSigPhase(pVideoOpt.phase);
        CSTVOUT_HDMI_IN_ENABLE(pVideoOpt.tvout_handle, 1);

        GetRxTvMode(&srcWidth, &srcHeight, &input_interlace);
        GetOutputTvMOde(pVideoOpt.tv_mode,&dstWidth, &dstHeight, &output_interlace);

        SrcRect.left = 0;
        SrcRect.right = srcWidth;
        SrcRect.top = 0;
        SrcRect.bottom = srcHeight;

        DstRect.left    = 0;
        DstRect.right   = srcWidth;
        DstRect.top     = 0;
        DstRect.bottom  = srcHeight;

        memset(&EncAttr, 0, sizeof(EncCpbAttr));

        EncAttr.CodecStd = pVideoOpt.input_codec;
        EncAttr.SubFrameSyncEn = 0;
        EncAttr.ReportPerField = 0;
        EncAttr.PicFormat.Format3D=0;//FORMAT_2D;
        EncAttr.PicFormat.IsInterlace= input_interlace;
        EncAttr.PicFormat.HorSample=1;
        EncAttr.PicFormat.VerSample=1;
        EncAttr.PicFormat.TopFieldFirst=1;
        EncAttr.PicFormat.VactSpace1=0;
        EncAttr.PicFormat.VactSpace2=0;
        EncAttr.PicFormat.PicWidth= srcWidth;
        EncAttr.PicFormat.PicHeight= srcHeight;
        EncAttr.PicFormat.FrameRate= VID_FRAME_RATE_3000;

        EncAttr.EncOpenParam.FrameRateInfo = VID_FRAME_RATE(60);
        EncAttr.EncOpenParam.RateCtrlMode = 1;
        EncAttr.EncOpenParam.StrictCbr = 0;
        EncAttr.EncOpenParam.BitRate = 15000;
        EncAttr.EncOpenParam.IntraRefresh = 0;
        EncAttr.EncOpenParam.QpY = 15;
        EncAttr.EncOpenParam.EnableAutoSkip = 0;
        EncAttr.EncOpenParam.GopSize = 120;
        EncAttr.EncOpenParam.MeSearchRangeX = 0;
        EncAttr.EncOpenParam.MeSearchRangeY = 0;
        EncAttr.EncOpenParam.Profile = 2;

        pVideoOpt.enc_handle = CSVID_OpenEncoder(pVideoOpt.dev_index, &EncAttr);
        if(NULL == pVideoOpt.enc_handle)
        {
            printf("Open encoder %d failed\n", pVideoOpt.dev_index);
            goto fail_open_dev;
        }

        //CSVID_SetEncLowLatencyMode(pVideoOpt.enc_handle, VID_LLE_PARTIAL_FRAME_MODE);

        sliceMode.sliceMode = 0;
        sliceMode.sliceSizeMode = 0;
        sliceMode.sliceSize = 1800;
        CSVID_SetEncSliceMode(pVideoOpt.enc_handle, &sliceMode);

        CSVID_SetSrcPosition(pVideoOpt.enc_handle,srcWidth, srcHeight, &SrcRect, input_interlace);
        CSVID_SetDstPosition(pVideoOpt.enc_handle,srcWidth, srcHeight, &DstRect, output_interlace);

        pVideoOpt.play_status = OPENED;
    }
    else if(pVideoOpt.device_type ==  VID_DEV_TYPE_CAPTURE)
    {

        CSHDMIRX_Open();
        CSHDMIRX_SetRepeaterMode(0);
        CSHDMIRX_SetHpd(1);
        CSHDMIRX_SetManSigPhase(pVideoOpt.phase);
        CSTVOUT_HDMI_IN_ENABLE(pVideoOpt.tvout_handle, 1);

        GetRxTvMode(&srcWidth, &srcHeight, &input_interlace);
        GetOutputTvMOde(pVideoOpt.tv_mode,&dstWidth, &dstHeight, &output_interlace);

        SrcRect.left = 0;
        SrcRect.right = srcWidth;
        SrcRect.top = 0;
        SrcRect.bottom = srcHeight;

        DstRect.left    = 0;
        DstRect.right   = dstWidth;
        DstRect.top     = 0;
        DstRect.bottom  = dstHeight;

        pVideoOpt.cap_handle = CSVID_OpenCapture(pVideoOpt.dev_index, srcWidth, srcHeight, input_interlace);
        pVideoOpt.cap_handle = CSVID_OpenCapture(pVideoOpt.dev_index, srcWidth, srcHeight, input_interlace);
        if(NULL == pVideoOpt.cap_handle)
        {
            printf("Open capture %d failed\n", pVideoOpt.dev_index);
            goto fail_open_dev;
        }

        CSVID_SetSrcPosition(pVideoOpt.cap_handle,srcWidth, srcHeight, &SrcRect, input_interlace);
        CSVID_SetDstPosition(pVideoOpt.cap_handle, dstWidth, dstHeight, &DstRect, output_interlace);

        pVideoOpt.play_status = OPENED;

    }
    else if(pVideoOpt.device_type ==  VID_DEV_TYPE_MULTI_CHANNEL)
    {

        CSHDMIRX_Open();
        CSHDMIRX_SetRepeaterMode(0);
        CSHDMIRX_SetHpd(1);
        CSHDMIRX_SetManSigPhase(pVideoOpt.phase);
        CSTVOUT_HDMI_IN_ENABLE(pVideoOpt.tvout_handle, 1);

        GetRxTvMode(&srcWidth, &srcHeight, &input_interlace);
        GetOutputTvMOde(pVideoOpt.tv_mode,&dstWidth, &dstHeight, &output_interlace);

        if(NULL == (pVideoOpt.dec_handle = CSVID_Open(pVideoOpt.dev_index)))
        {
            printf("Open deooder %d failed\n", pVideoOpt.dev_index);
            goto fail_open_dev;
        }

        pVideoOpt.cap_handle = CSVID_OpenCapture(pVideoOpt.dev_index, srcWidth, srcHeight, input_interlace);
        if(NULL == pVideoOpt.cap_handle)
        {
            printf("Open capture %d failed\n", pVideoOpt.dev_index);
            goto fail_open_dev;
        }

        memset(&EncAttr, 0, sizeof(EncCpbAttr));

        EncAttr.CodecStd = pVideoOpt.input_codec;
        EncAttr.SubFrameSyncEn = 0;
        EncAttr.PicFormat.Format3D=0;//FORMAT_2D;
        EncAttr.PicFormat.IsInterlace= input_interlace;
        EncAttr.PicFormat.HorSample=1;
        EncAttr.PicFormat.VerSample=1;
        EncAttr.PicFormat.TopFieldFirst=1;
        EncAttr.PicFormat.VactSpace1=0;
        EncAttr.PicFormat.VactSpace2=0;
        EncAttr.PicFormat.PicWidth= srcWidth;
        EncAttr.PicFormat.PicHeight= srcHeight;
        EncAttr.PicFormat.FrameRate= VID_FRAME_RATE_3000;

        EncAttr.EncOpenParam.FrameRateInfo = VID_FRAME_RATE(60);
        EncAttr.EncOpenParam.RateCtrlMode = 1;
        EncAttr.EncOpenParam.BitRate = 10000;
        EncAttr.EncOpenParam.IntraRefresh = 0;
        EncAttr.EncOpenParam.QpY = 20;
        EncAttr.EncOpenParam.EnableAutoSkip = 0;
        EncAttr.EncOpenParam.GopSize = 120;
        EncAttr.EncOpenParam.MeSearchRangeX = 0;
        EncAttr.EncOpenParam.MeSearchRangeY = 0;
        EncAttr.EncOpenParam.Profile = 2;

        pVideoOpt.enc_handle = CSVID_OpenEncoder(pVideoOpt.dev_index, &EncAttr);
        if(NULL == pVideoOpt.enc_handle)
        {
            printf("Open encoder %d failed\n", pVideoOpt.dev_index);
            goto fail_open_dev;
        }

        CSVID_SetStreamType(pVideoOpt.dec_handle, pVideoOpt.input_codec);
        CSVID_SetDecoderReorder(pVideoOpt.dec_handle, 1);
        CSVID_EnablePTSSync(pVideoOpt.dec_handle);
        CSVID_SetVideoWithAudio(pVideoOpt.dec_handle, 0);
        CSVID_SetLowLatencyMode(pVideoOpt.dec_handle, VID_LLD_STREAM_LOW_DELAY_MODE, 10);

        //CSVID_SyncNotify(pVideoOpt.dec_handle, synccall, 1000000, 1);

        SrcRect.left = 0;
        SrcRect.right = srcWidth;
        SrcRect.top = 0;
        SrcRect.bottom = srcHeight;

        DstRect.left    = 0;
        DstRect.right   = 960;
        DstRect.top     = 0;
        DstRect.bottom  = 600;

        CSVID_SetSrcPosition(pVideoOpt.cap_handle, srcWidth, srcHeight, &SrcRect, input_interlace);
        CSVID_SetDstPosition(pVideoOpt.cap_handle, dstWidth, dstHeight, &DstRect, output_interlace);

        CSVID_SetSrcPosition(pVideoOpt.enc_handle, srcWidth, srcHeight, &SrcRect, input_interlace);
        CSVID_SetDstPosition(pVideoOpt.enc_handle, srcWidth, srcHeight, &SrcRect, output_interlace);

        DstRect.left    = 960;
        DstRect.right   = 1920;
        DstRect.top     = 0;
        DstRect.bottom  = 600;

        CSVID_SetSrcPosition(pVideoOpt.dec_handle, srcWidth, srcHeight, &SrcRect, input_interlace);
        CSVID_SetDstPosition(pVideoOpt.dec_handle, dstWidth, dstHeight, &DstRect, output_interlace);

        pVideoOpt.play_status = OPENED;
    }
    else if(pVideoOpt.device_type ==  VID_DEV_TYPE_TRANSCODING)
    {
        EncSliceMode sliceMode;

        GetOutputTvMOde(pVideoOpt.tv_mode, &dstWidth, &dstHeight, &output_interlace);

        memset(&EncAttr, 0, sizeof(EncCpbAttr));

        EncAttr.CodecStd = pVideoOpt.output_codec;//VID_STREAM_TYPE_H264;
        EncAttr.SubFrameSyncEn = 0;
        EncAttr.ReportPerField = 0;
        EncAttr.PicFormat.Format3D=0;//FORMAT_2D;
        EncAttr.PicFormat.IsInterlace= 0;
        EncAttr.PicFormat.HorSample=1;
        EncAttr.PicFormat.VerSample=1;
        EncAttr.PicFormat.TopFieldFirst=1;
        EncAttr.PicFormat.VactSpace1=0;
        EncAttr.PicFormat.VactSpace2=0;
        EncAttr.PicFormat.PicWidth= pVideoOpt.width;
        EncAttr.PicFormat.PicHeight= pVideoOpt.height;
        EncAttr.PicFormat.FrameRate= VID_FRAME_RATE_3000;

        EncAttr.EncOpenParam.FrameRateInfo = VID_FRAME_RATE(30);
        EncAttr.EncOpenParam.RateCtrlMode = 1;
        EncAttr.EncOpenParam.StrictCbr = 0;
        EncAttr.EncOpenParam.BitRate = 10000;
        EncAttr.EncOpenParam.IntraRefresh = 0;
        EncAttr.EncOpenParam.QpY = 20;
        EncAttr.EncOpenParam.EnableAutoSkip = 0;
        EncAttr.EncOpenParam.GopSize = 120;
        EncAttr.EncOpenParam.MeSearchRangeX = 0;
        EncAttr.EncOpenParam.MeSearchRangeY = 0;
        EncAttr.EncOpenParam.Profile = 2;

        if(CSAPI_FAILED == CSVID_OpenTranscoding(pVideoOpt.dev_index, &EncAttr, &pVideoOpt.dec_handle, &pVideoOpt.enc_handle))
        {
            printf("Open transcoding %d failed\n", pVideoOpt.dev_index);
            goto fail_open_dev;
        }

        sliceMode.sliceMode = 0;
        sliceMode.sliceSizeMode = 0;
        sliceMode.sliceSize = 1800;
        CSVID_SetEncSliceMode(pVideoOpt.enc_handle, &sliceMode);

        CSVID_SetStreamType(pVideoOpt.dec_handle, pVideoOpt.input_codec);
        CSVID_SetDecoderReorder(pVideoOpt.dec_handle, 1);
        CSVID_DisablePTSSync(pVideoOpt.dec_handle);
        CSVID_SetVideoWithAudio(pVideoOpt.dec_handle, 0);

        pVideoOpt.play_status = OPENED;
    }
    else
    {
        printf("Invalid device %d\n", pVideoOpt.dev_index);
        goto fail_open_dev;
    }

    if(pVideoOpt.device_type == VID_DEV_TYPE_DECODER)
    {
        if (pVideoOpt.input_codec != VID_STREAM_TYPE_YUV)
        {
            if(0 != pthread_create(&pVideoOpt.task, NULL, write_stream_to_decoder, &pVideoOpt))
            //if(0 != pthread_create(&pVideoOpt.task, NULL, write_stream_to_decoder_by_frame, &pVideoOpt))
            {
                printf("Create task failed\n");
                goto fail_task;
            }

            pthread_create(&pVideoOpt.read_enc_task, NULL, read_yuv_from_decoder, &pVideoOpt);
        }
        else
        {
            if(0 != pthread_create(&pVideoOpt.task, NULL, write_yuv_to_display, &pVideoOpt))
            {
                printf("Create task failed\n");
                goto fail_task;
            }
        }
    }
    else if(pVideoOpt.device_type == VID_DEV_TYPE_DECODER_SLOW)
    {
        if (pVideoOpt.input_codec != VID_STREAM_TYPE_YUV)
        {
            if(0 != pthread_create(&pVideoOpt.task, NULL, write_stream_to_decoder_by_frame, &pVideoOpt))
            {
                printf("Create task failed\n");
                goto fail_task;
            }

            //pthread_create(&pVideoOpt.read_enc_task, NULL, read_yuv_from_decoder, &pVideoOpt);
        }
        //else
        //{
        //    if(0 != pthread_create(&pVideoOpt.task, NULL, write_yuv_to_display, &pVideoOpt))
        //    {
        //        printf("Create task failed\n");
        //        goto fail_task;
        //    }
        //}
    }
    else if(pVideoOpt.device_type == VID_DEV_TYPE_ENCODER)
    {
        if(0 != pthread_create(&pVideoOpt.task, NULL, read_stream_from_encoder, &pVideoOpt))
        {
            printf("Create task failed\n");
            goto fail_task;
        }
    }
    else if(pVideoOpt.device_type == VID_DEV_TYPE_MULTI_CHANNEL)
    {
        if(0 != pthread_create(&pVideoOpt.task, NULL, do_from_encoder_to_decoder, &pVideoOpt))
        {
            printf("Create task failed\n");
            goto fail_task;
        }
    }
    else if(pVideoOpt.device_type == VID_DEV_TYPE_TRANSCODING)
    {
        /*if(0 != pthread_create(&pVideoOpt.task, NULL, transcoding_proc, &pVideoOpt)){
            printf("Create task failed\n");
            goto fail_task;
        }*/

        if(0 != pthread_create(&pVideoOpt.task, NULL, write_stream_to_decoder, &pVideoOpt))
        {
            printf("Create task failed\n");
            goto fail_task;
        }

        if(0 != pthread_create(&pVideoOpt.read_enc_task, NULL, read_stream_from_encoder, &pVideoOpt))
        {
            printf("Create task failed\n");
            goto fail_task;
        }
    }

    signal(SIGINT, abort_quit);

    while(1)
    {
        char cmd;

        show_command();
        cmd = get_command();
        if(0 != process_command(&pVideoOpt, cmd))
        {
            exit(0);
        }
        usleep(500000);
    }
fail_task:
    if(pVideoOpt.device_type == VID_DEV_TYPE_DECODER)
    {
        CSVID_Close(pVideoOpt.dec_handle);
    }
    else if(pVideoOpt.device_type == VID_DEV_TYPE_DECODER_SLOW)
    {
        CSVID_Close(pVideoOpt.dec_handle);
    }
    else if(pVideoOpt.device_type == VID_DEV_TYPE_ENCODER)
    {
        CSVID_CloseEncoder(pVideoOpt.dec_handle);
    }  
    else if(pVideoOpt.device_type == VID_DEV_TYPE_TRANSCODING)
    {
        CSVID_CloseTranscoding(pVideoOpt.dec_handle,pVideoOpt.enc_handle);
    }
    
fail_open_dev:
    CSVID_Term();
fail_init_videofw:
    CSTVOUT_Close(pVideoOpt.tvout_handle);
fail_open_tvout:
    return -1;
}
static void* transcoding_proc(void *params)
{
#define DIR_SIZE 16
    unsigned int read_lens;
    unsigned int cpb_bufsize = 0;
    unsigned int dir_bufsize = 0;
    unsigned int cpb_data_size = 0;
    int dir_data_size = 0;
    int end_of_frame = 0;
    unsigned int   recv_size = 0;
    unsigned long long pts = 0;
    unsigned char  dec_buf[1048576*2+1024];
    unsigned char  enc_buf[1048576*4+1024];
    VideoOpt_t *pVideoOpt = (VideoOpt_t*)params;

    if(NULL == (pVideoOpt->input_fp = fopen(pVideoOpt->input_file,"rb")))
    {
        printf("%s file open failed\n", pVideoOpt->input_file);
        return NULL;
    }

    if(NULL == (pVideoOpt->output_fp = fopen(pVideoOpt->output_file,"wb")))
    {
        printf("%s file open failed\n", pVideoOpt->output_file);
        fclose(pVideoOpt->input_fp);
        return NULL;
    }

    while(0 == sem_wait(&pVideoOpt->lock))
    {
        sem_post(&pVideoOpt->lock);

        //VID_GetCpbBufferSize(pVideoOpt->dec_handle, VID_DEV_TYPE_DECODER, &cpb_bufsize);
        //VID_GetDirBufferSize(pVideoOpt->dec_handle, VID_DEV_TYPE_DECODER, &dir_bufsize);
        //printf("Decoder cpb_bufsize:%d  dir_bufsize:%d\n", cpb_bufsize, dir_bufsize);
        if((cpb_bufsize > 0) && (dir_bufsize/DIR_SIZE >= 2))
        {
            if(cpb_bufsize < sizeof(dec_buf))
                read_lens = fread(dec_buf, 1, cpb_bufsize, pVideoOpt->input_fp);
            else
                read_lens = fread(dec_buf, 1, sizeof(dec_buf) - 1,pVideoOpt->input_fp);

            if(read_lens > 0)
                CSVID_WritePFMDataWithPTS(pVideoOpt->dec_handle, dec_buf, read_lens, 0x0);
            else
                fseek(pVideoOpt->input_fp, 0, SEEK_SET);
        }
        //VID_GetCpbDataSize(pVideoOpt->enc_handle, VID_DEV_TYPE_ENCODER, &cpb_data_size);
        //VID_GetDirDataSize(pVideoOpt->enc_handle, VID_DEV_TYPE_ENCODER, &dir_data_size);
        if(cpb_data_size > 0)
            CSVID_ReadEncChunkWithPts(pVideoOpt->enc_handle, enc_buf, sizeof(enc_buf), &recv_size, &pts, &end_of_frame);
        if(recv_size > 0)
            fwrite(enc_buf, recv_size, 1, pVideoOpt->output_fp);
        recv_size = 0;
    }

    return NULL;
}
static void* write_stream_to_decoder(void *params)
{
    unsigned char  read_buf[READFILE_SIZE+100];
    int random_size = 0;
    int read_len = 0;
    int read_finish = 0;
    VideoOpt_t *pVideoOpt = (VideoOpt_t*)params;

    pVideoOpt->input_fp = fopen(pVideoOpt->input_file,"rb");
    if(pVideoOpt->input_fp == NULL)
    {
        printf("======>%s file open failed\n", pVideoOpt->input_file);
        return NULL;
    }

    while(0 == sem_wait(&pVideoOpt->lock))
    {
        sem_post(&pVideoOpt->lock);
        if(pVideoOpt->restart)
        {
            fseek(pVideoOpt->input_fp,0L, SEEK_SET);
            random_size = random(READFILE_SIZE);
            if ((read_len=fread(read_buf,1,random_size,pVideoOpt->input_fp))==0)
            {
                fseek(pVideoOpt->input_fp,0L, SEEK_SET);
                printf("1read_len = %d ,===no data \n",read_len);
                return NULL;
            }
            printf("Replay ...\n");
            pVideoOpt->restart = 0;
        }

        if(CSAPI_SUCCEED == CSVID_WritePFMData(pVideoOpt->dec_handle, read_buf, read_len,0x0))
        {
            if(read_finish)
            {
                break;
            }
            else
            {
                random_size = random(READFILE_SIZE);
                while ((read_len=fread(read_buf,1,random_size,pVideoOpt->input_fp))==0)
                {
                    fseek(pVideoOpt->input_fp,0L, SEEK_SET);
                    if(pVideoOpt->loop == 0)
                    {
                        read_finish = 1;
                        break;
                    }
                    //printf("read_len = %d ,===no data \n",read_len);
                    random_size = random(READFILE_SIZE);
                    usleep(100*1000);
                }
            }
        }
        else
        {
            usleep(100*1000);
        }
    }

    //fclose(pVideoOpt->input_fp);
    //pVideoOpt->input_fp = NULL;

    return NULL;
}

static void* write_stream_to_decoder_by_frame(void *params)
{
#define WRITE_FRAME_DELAY (50 * 1000)
#define FRAME_DATA_LENGTH (1 * 1000 * 1000)

    unsigned char  p_frameDataBuf[FRAME_DATA_LENGTH];
    char p_ch_curDir[256] = "";
    char p_ch_frameFileName[256] = "";
    char p_ch_frameUrl[512] = "";
    FILE *fp = NULL;
    int s32_dataLength = 0;
    
    int random_size = 0;
    int read_len = 0;
    int read_finish = 0;
    char * p = NULL;
    VideoOpt_t *pVideoOpt = (VideoOpt_t*)params;

    strcpy(p_ch_curDir,pVideoOpt->input_file);
    p = strrchr(p_ch_curDir,'/');
    if(p)
    {
        *(p + 1) = '\0';
    }
    else
    {
        p_ch_curDir[0] = '\0';
    }
    
    pVideoOpt->input_fp = fopen(pVideoOpt->input_file,"rb");
    if(pVideoOpt->input_fp == NULL)
    {
        printf("======>%s file open failed\n", pVideoOpt->input_file);
        return NULL;
    }

    
    while(fgets(p_ch_frameFileName,256,pVideoOpt->input_fp) && !sem_wait(&pVideoOpt->lock)) 
    {
        sprintf(p_ch_frameUrl,"%s%s",p_ch_curDir,p_ch_frameFileName);
        *(p_ch_frameUrl + strlen(p_ch_frameUrl) - 1) = '\0';
        sem_post(&pVideoOpt->lock);
        printf("open file [%s]\n",p_ch_frameUrl);
        fp = fopen(p_ch_frameUrl,"rb");
        if(!fp)
        {
            printf("open failed.\n");
            continue;
        }
        s32_dataLength = fread(p_frameDataBuf,1,FRAME_DATA_LENGTH,fp);
        if(s32_dataLength > 0)
        {
            printf("Get data [%d k]\n",s32_dataLength/1000);
            while(CSVID_WritePFMData(pVideoOpt->dec_handle, p_frameDataBuf, s32_dataLength,0))
            {
                printf("write data failed.\n");
                usleep(100 * 1000);
            }
        }
        fclose(fp);
        usleep(WRITE_FRAME_DELAY);
        
    }

    fclose(pVideoOpt->input_fp);
    pVideoOpt->input_fp = NULL;

    return NULL;
}

static void* write_yuv_to_display(void *params)
{
    VideoOpt_t *pVideoOpt = (VideoOpt_t*)params;
    int read_len;
    CSVID_Yuv_Frame yuv_frame;
    unsigned long long pts = 0;

    pVideoOpt->input_fp = fopen(pVideoOpt->input_file,"rb");
    if(pVideoOpt->input_fp == NULL)
    {
        printf("======>%s file open failed\n", pVideoOpt->input_file);
        return NULL;
    }

    if ((pVideoOpt->width < 0) || (pVideoOpt->width > 1920) ||
            (pVideoOpt->height < 0) || (pVideoOpt->height > 1088))
    {
        printf("======>invalid YUV resolution: %d x %d\n", pVideoOpt->width, pVideoOpt->height);
        return NULL;
    }

    while(0 == sem_wait(&pVideoOpt->lock))
    {
        sem_post(&pVideoOpt->lock);
        if(pVideoOpt->restart)
        {
            fseek(pVideoOpt->input_fp,0L, SEEK_SET);
            printf("Replay ...\n");
            pVideoOpt->restart = 0;
        }

        yuv_frame.p_y_data = YUVTempBuffer;
        yuv_frame.y_stride = pVideoOpt->width;
        yuv_frame.cbcr_interleave = 0;
        yuv_frame.p_cb_data = YUVTempBuffer + pVideoOpt->width * pVideoOpt->height;
        if (yuv_frame.cbcr_interleave)
        {
            yuv_frame.p_cr_data = yuv_frame.p_cb_data + 1;
            yuv_frame.cbcr_stride = pVideoOpt->width;
        }
        else
        {
            yuv_frame.p_cr_data = yuv_frame.p_cb_data + pVideoOpt->width * pVideoOpt->height / 4;
            yuv_frame.cbcr_stride = pVideoOpt->width / 2;
        }
        yuv_frame.width = pVideoOpt->width;
        yuv_frame.height = pVideoOpt->height;

        //load yuv from file
        read_len = fread(yuv_frame.p_y_data, 1, pVideoOpt->width * pVideoOpt->height, pVideoOpt->input_fp);
        read_len = fread(yuv_frame.p_cb_data, 1, pVideoOpt->width * pVideoOpt->height / 4, pVideoOpt->input_fp);
        read_len = fread(yuv_frame.p_cr_data, 1, pVideoOpt->width * pVideoOpt->height / 4, pVideoOpt->input_fp);

        if (read_len != pVideoOpt->width * pVideoOpt->height / 4)
        {
            fseek(pVideoOpt->input_fp,0L, SEEK_SET);
            continue;
        }

        CSVID_WriteYuvFrameWithPTS(pVideoOpt->dec_handle, &yuv_frame, pts);
        usleep(30*1000);
    }

    return NULL;
}

static void* read_stream_from_encoder(void *params)
{
    unsigned int recv_size = 0;
    unsigned long long  pts = 0;
    unsigned char  read_buf[1048576*4+1024];
    VideoOpt_t *pVideoOpt = (VideoOpt_t*)params;

    pVideoOpt->output_fp = fopen(pVideoOpt->output_file, "wb");
    if(pVideoOpt->input_fp)
    {
        while(0 == sem_wait(&pVideoOpt->lock))
        {
            int end_of_frame;
            sem_post(&pVideoOpt->lock);
            //CSVID_ReadEncOneFrameWithPTS(pVideoOpt->enc_handle, read_buf,sizeof(read_buf), &recv_size, &pts);
            CSVID_ReadEncChunkWithPts(pVideoOpt->enc_handle, read_buf,sizeof(read_buf), &recv_size, &pts, &end_of_frame);
            if(recv_size > 0)
            {
                fwrite(read_buf, recv_size, 1, pVideoOpt->output_fp);
                fsync(fileno(pVideoOpt->output_fp));
                //printf("read encoder data size:%d\n", recv_size);
                //usleep(100*1000);
            }
            else
            {
                usleep(100*1000);
            }

        }
    }
    return NULL;
}

static void *do_from_encoder_to_decoder(void *params)
{
    unsigned int event;
    CSAPI_RESULT ret;
    struct timeval time_out;
    unsigned int recv_size = 0;
    unsigned long long pts;
    unsigned char  read_buf[1048576*4+1024];
    VideoOpt_t *pVideoOpt = (VideoOpt_t*)params;

    while(0 == sem_wait(&pVideoOpt->lock))
    {
        sem_post(&pVideoOpt->lock);

        event = 0;
        time_out.tv_sec = 0;
        time_out.tv_usec = 200*1000;
        if((CSAPI_FAILED == CSVID_Event(pVideoOpt->enc_handle, &event, time_out)) || (event == 0))
        {
            //usleep(200*1000);
            continue;
        }

        if(event & VID_INT_ENC_FRAME_DONE)
        {
            do
            {
                ret = CSVID_ReadEncOneFrameWithPTS(pVideoOpt->enc_handle, read_buf,sizeof(read_buf), &recv_size, &pts);
                if(recv_size > 0)
                {
                    CSVID_WritePFMDataWithPTS(pVideoOpt->dec_handle, read_buf, recv_size, pts);
                }
            }
            while(ret == CSAPI_SUCCEED);
        }
    }
    return NULL;
}
unsigned int DumpYuvFrame(VideoYUVInfoT *pYuvInfo)
{
    unsigned int i=0,j=0,m=0,n=0;

    FILE *stream=NULL;
    stream=fopen("./test.yuv", "wb");
    if(stream==NULL)
    {
        printf("dump open file fail\r\n");
        return -1;
    }
    for(i=0; i<pYuvInfo->PicHeight; i++)
        memcpy((unsigned char *)((unsigned int)YUVTempBuffer+(unsigned int)pYuvInfo->PicWidth*i), pYuvInfo->Yaddr+i*pYuvInfo->LinePitch, pYuvInfo->PicWidth);

    for(i=0; i<(pYuvInfo->PicHeight>>1/*pYuvInfo->SrcIsYuv420*/); i++)
    {
        memcpy((unsigned char *)((unsigned int)YUVTempBuffer1), pYuvInfo->Caddr+i*pYuvInfo->LinePitch, pYuvInfo->PicWidth);

        for(j=0; j<(pYuvInfo->PicWidth); j++)
        {
            if(j%2)
            {
                YUVTempBuffer[(unsigned int)pYuvInfo->PicWidth*(pYuvInfo->PicHeight+((pYuvInfo->PicHeight>>1/*pYuvInfo->SrcIsYuv420*/)>>1))+(n++)]=YUVTempBuffer1[j];

            }
            else
            {
                YUVTempBuffer[(unsigned int)pYuvInfo->PicWidth*pYuvInfo->PicHeight+(m++)]=YUVTempBuffer1[j];
            }
        }
    }
    fwrite((void*)YUVTempBuffer,1,(unsigned int)pYuvInfo->PicWidth*(pYuvInfo->PicHeight+(pYuvInfo->PicHeight>>1/*pYuvInfo->SrcIsYuv420*/)),stream);
    fclose(stream);
    return 0;
}

static void *read_yuv_from_decoder(void *params)
{
    FILE *fp;
    unsigned int event;
    struct timeval time_out;
    VideoOpt_t *pVideoOpt = (VideoOpt_t*)params;

    fp = fopen("./test.yuv", "wb");
    if(NULL == fp)
        return NULL;
    while(1)
    {
        event = 0;
        time_out.tv_sec = 0;
        time_out.tv_usec = 200*1000;
        if((CSAPI_FAILED == CSVID_Event(pVideoOpt->dec_handle, &event, time_out)) || (event == 0))
        {
            //usleep(200*1000);
            continue;
        }

        if(event & VID_INT_FRAME_OUTPUT)
        {
            VideoYUVInfoT YuvInfo;
            memset(&YuvInfo, 0, sizeof(YuvInfo));
            CSVID_QueryYUVInfo(pVideoOpt->dec_handle, &YuvInfo);
            DumpYuvFrame(&YuvInfo);
            CSVID_ReleaseYUVInfo(pVideoOpt->dec_handle, &YuvInfo);
            printf("get yuv succseefly\n");
            break;
        }
    }
    return NULL;
}

static void show_video_help(void)
{
    printf("\t-D <DEC0 || DEC0_STEP|| ENC0 || MULTI || TRANSCODING > select device type and device index\n");
    printf("\t-I input file\n");
    printf("\t-O output file > encoder or transcoding output file\n");
    printf("\t-W transcoding source width \n");
    printf("\t-H transcoding source height \n");
    printf("\t-R <720P60 || 1080P30 || 1080P60> determined resolution\n");
    printf("\t-C <H264 || VC1 || MPEG2 || MPEG4 || H263 || DIV3 || RV || AVS || MJPG || THO || VP3 || VP8 || MVC || YUV> select codec type\n");
}
static void show_command(void)
{
    printf("Press under key:\n");
    printf("\t\"r\" play\n");
    printf("\t\"s\" stop\n");
    printf("\t\"q\" quit\n");
    printf("cmd:");
}
static char get_command(void)
{
    unsigned int val = 0;
    char ch = 0x00;
    val = getchar();
    ch = val & 0xff;
    return ch;
}

static int process_command(VideoOpt_t *pVideoOpt, char cmd)
{
    EncCpbAttr EncAttr;
    CSVID_Rect SrcRect, DstRect;
    unsigned int input_interlace = 0, output_interlace = 0;
    unsigned int dstHeight=0, dstWidth=0, srcWidth = 0, srcHeight = 0;
    CSVID_Rect dst;
    CSVID_Crop src;
    static unsigned char step=0;
    switch(cmd)
    {
    case 'r':
        if(pVideoOpt->device_type == VID_DEV_TYPE_DECODER)
        {
            if(pVideoOpt->play_status == OPENED || pVideoOpt->play_status == STOP)
            {
                pVideoOpt->restart = 1;
                CSVID_SetStartPTS(pVideoOpt->dec_handle, 0);
                CSVID_Play(pVideoOpt->dec_handle);
                pVideoOpt->play_status = PLAY;
                sem_post(&pVideoOpt->lock);
            }
        }
        if(pVideoOpt->device_type == VID_DEV_TYPE_DECODER_SLOW)
        {
            if(pVideoOpt->play_status == OPENED || pVideoOpt->play_status == STOP)
            {
                //CSVID_SetStartPTS(pVideoOpt->dec_handle, 0);
                CSVID_Play(pVideoOpt->dec_handle);
                pVideoOpt->play_status = PLAY;
                sem_post(&pVideoOpt->lock);
            }
        }
        else if(pVideoOpt->device_type == VID_DEV_TYPE_ENCODER)
        {
            if(pVideoOpt->play_status == OPENED || pVideoOpt->play_status == STOP)
            {
                pVideoOpt->restart = 1;
                CSVID_PlayEncoder(pVideoOpt->enc_handle);
                pVideoOpt->play_status = PLAY;
                sem_post(&pVideoOpt->lock);
            }
        }
        else if(pVideoOpt->device_type == VID_DEV_TYPE_CAPTURE)
        {
            if(pVideoOpt->play_status == OPENED || pVideoOpt->play_status == STOP)
            {
                CSVID_PlayCapture(pVideoOpt->cap_handle);
                pVideoOpt->play_status = PLAY;
                //sem_post(&pVideoOpt->lock);
            }
        }
        else if(pVideoOpt->device_type == VID_DEV_TYPE_MULTI_CHANNEL)
        {
            if(pVideoOpt->play_status == OPENED || pVideoOpt->play_status == STOP)
            {
                pVideoOpt->restart = 1;
                CSVID_PlayCapture(pVideoOpt->cap_handle);
                CSVID_PlayEncoder(pVideoOpt->enc_handle);
                CSVID_SetStartPTS(pVideoOpt->dec_handle, 0);
                CSVID_Play(pVideoOpt->dec_handle);
                pVideoOpt->play_status = PLAY;
                sem_post(&pVideoOpt->lock);
            }
        }
        else if(pVideoOpt->device_type == VID_DEV_TYPE_TRANSCODING)
        {
            if(pVideoOpt->play_status == OPENED || pVideoOpt->play_status == STOP)
            {
                pVideoOpt->restart = 1;
                CSVID_SetStartPTS(pVideoOpt->dec_handle, 0);
                CSVID_PlayTranscoding(pVideoOpt->dec_handle, pVideoOpt->enc_handle);
                pVideoOpt->play_status = PLAY;
                sem_post(&pVideoOpt->lock);
            }
        }
        break;
    case 's':
        if(pVideoOpt->device_type == VID_DEV_TYPE_DECODER)
        {
            if(pVideoOpt->play_status == PLAY)
            {
                sem_wait(&pVideoOpt->lock);
                CSVID_Stop(pVideoOpt->dec_handle);
                pVideoOpt->play_status = STOP;
            }
        }
        else if(pVideoOpt->device_type == VID_DEV_TYPE_DECODER_SLOW)
        {
            if(pVideoOpt->play_status == PLAY)
            {
                sem_wait(&pVideoOpt->lock);
                CSVID_Stop(pVideoOpt->dec_handle);
                pVideoOpt->play_status = STOP;
            }
        }
        else if(pVideoOpt->device_type == VID_DEV_TYPE_ENCODER)
        {
            if(pVideoOpt->play_status == PLAY)
            {
                sem_wait(&pVideoOpt->lock);
                CSVID_StopEncoder(pVideoOpt->enc_handle);
                pVideoOpt->play_status = STOP;
            }
        }
        else if(pVideoOpt->device_type == VID_DEV_TYPE_CAPTURE)
        {
            if(pVideoOpt->play_status == PLAY)
            {
                //sem_wait(&pVideoOpt->lock);
                CSVID_StopCapture(pVideoOpt->cap_handle);
                pVideoOpt->play_status = STOP;
            }
        }
        else if(pVideoOpt->device_type == VID_DEV_TYPE_MULTI_CHANNEL)
        {
            if(pVideoOpt->play_status == PLAY)
            {
                sem_wait(&pVideoOpt->lock);
                CSVID_StopCapture(pVideoOpt->cap_handle);
                CSVID_StopEncoder(pVideoOpt->enc_handle);
                CSVID_Stop(pVideoOpt->dec_handle);
                pVideoOpt->play_status = STOP;
            }
        }
        else if(pVideoOpt->device_type == VID_DEV_TYPE_TRANSCODING)
        {
            if(pVideoOpt->play_status == PLAY)
            {
                sem_wait(&pVideoOpt->lock);
                CSVID_StopTranscoding(pVideoOpt->dec_handle, pVideoOpt->enc_handle);
                pVideoOpt->play_status = STOP;
            }
        }
        break;
    case 'o':
        if(pVideoOpt->device_type == VID_DEV_TYPE_DECODER)
        {
            if(pVideoOpt->play_status == CLOSED)
            {
                if(NULL == (pVideoOpt->dec_handle = CSVID_Open(pVideoOpt->dev_index)))
                {
                    printf("%d %s Open deooder %d failed\n",__LINE__, __FUNCTION__, pVideoOpt->dev_index);
                    return 0;
                }

                GetOutputTvMOde(pVideoOpt->tv_mode,&dstWidth, &dstHeight, &output_interlace);

                DstRect.left = 0;
                DstRect.right = dstWidth;
                DstRect.top   = 0;
                DstRect.bottom = dstHeight;

                CSVID_SetStreamType(pVideoOpt->dec_handle, pVideoOpt->input_codec);
                CSVID_SetDecoderReorder(pVideoOpt->dec_handle, 1);
                CSVID_EnablePTSSync(pVideoOpt->dec_handle);
                CSVID_SetVideoWithAudio(pVideoOpt->dec_handle, 0);
                CSVID_SyncNotify(pVideoOpt->dec_handle, synccall, 1000000, 1);

                CSVID_SetDstPosition(pVideoOpt->dec_handle, dstWidth, dstHeight, &DstRect, output_interlace);

                pVideoOpt->play_status = OPENED;

                sem_init(&pVideoOpt->lock, 0, 0);

                if (pVideoOpt->input_codec != VID_STREAM_TYPE_YUV)
                {
                    if(0 != pthread_create(&pVideoOpt->task, NULL, write_stream_to_decoder, pVideoOpt))
                    {
                        printf("%d %s Create task failed\n", __LINE__, __FUNCTION__);
                    }
                }
                else
                {
                    if(0 != pthread_create(&pVideoOpt->task, NULL, write_yuv_to_display, pVideoOpt))
                    {
                        printf("%d %s Create task failed\n", __LINE__, __FUNCTION__);
                    }
                }
            }
        }
        else if(pVideoOpt->device_type == VID_DEV_TYPE_ENCODER)
        {
            if(pVideoOpt->play_status == CLOSED)
            {

                //CSHDMIRX_Open();
                //CSHDMIRX_SetManSigPhase(pVideoOpt->phase);
                //CSTVOUT_HDMI_IN_ENABLE(pVideoOpt->tvout_handle, 1);

                GetRxTvMode(&srcWidth, &srcHeight, &input_interlace);
                GetOutputTvMOde(pVideoOpt->tv_mode,&dstWidth, &dstHeight, &output_interlace);

                SrcRect.left = 0;
                SrcRect.right = srcWidth;
                SrcRect.top = 0;
                SrcRect.bottom = srcHeight;

                DstRect.left    = 0;
                DstRect.right   = srcWidth;
                DstRect.top     = 0;
                DstRect.bottom  = srcHeight;

                memset(&EncAttr, 0, sizeof(EncCpbAttr));

                EncAttr.CodecStd = pVideoOpt->input_codec;
                EncAttr.SubFrameSyncEn = 0;
                EncAttr.PicFormat.Format3D=0;//FORMAT_2D;
                EncAttr.PicFormat.IsInterlace= input_interlace;
                EncAttr.PicFormat.HorSample=1;
                EncAttr.PicFormat.VerSample=1;
                EncAttr.PicFormat.TopFieldFirst=1;
                EncAttr.PicFormat.VactSpace1=0;
                EncAttr.PicFormat.VactSpace2=0;
                EncAttr.PicFormat.PicWidth= srcWidth;
                EncAttr.PicFormat.PicHeight= srcHeight;
                EncAttr.PicFormat.FrameRate= VID_FRAME_RATE_3000;

                EncAttr.EncOpenParam.FrameRateInfo = VID_FRAME_RATE(30);
                EncAttr.EncOpenParam.RateCtrlMode = 1;
                EncAttr.EncOpenParam.BitRate = 5000;
                EncAttr.EncOpenParam.IntraRefresh = 0;
                EncAttr.EncOpenParam.QpY = 20;
                EncAttr.EncOpenParam.EnableAutoSkip = 0;
                EncAttr.EncOpenParam.GopSize = 64;
                EncAttr.EncOpenParam.MeSearchRangeX = 0;
                EncAttr.EncOpenParam.MeSearchRangeY = 2;
                EncAttr.EncOpenParam.Profile = 2;

                pVideoOpt->enc_handle = CSVID_OpenEncoder(pVideoOpt->dev_index, &EncAttr);
                if(NULL == pVideoOpt->enc_handle)
                {
                    printf("%d %s Open encoder %d failed\n",__LINE__, __FUNCTION__, pVideoOpt->dev_index);
                    return -1;
                }

                CSVID_SetSrcPosition(pVideoOpt->enc_handle,srcWidth, srcHeight, &SrcRect, input_interlace);
                CSVID_SetDstPosition(pVideoOpt->enc_handle,srcWidth, srcHeight, &DstRect, output_interlace);

                sem_init(&pVideoOpt->lock, 0, 0);

                if(0 != pthread_create(&pVideoOpt->task, NULL, read_stream_from_encoder, pVideoOpt))
                {
                    printf("%d %s Create task failed\n", __LINE__, __FUNCTION__);
                }
                pVideoOpt->play_status = OPENED;
            }
        }
        else if(pVideoOpt->device_type == VID_DEV_TYPE_CAPTURE)
        {
            if(pVideoOpt->play_status == CLOSED)
            {

                //CSHDMIRX_Open();
                //CSHDMIRX_SetManSigPhase(pVideoOpt->phase);
                //CSTVOUT_HDMI_IN_ENABLE(pVideoOpt->tvout_handle, 1);

                GetRxTvMode(&srcWidth, &srcHeight, &input_interlace);
                GetOutputTvMOde(pVideoOpt->tv_mode,&dstWidth, &dstHeight, &output_interlace);

                SrcRect.left = 0;
                SrcRect.right = srcWidth;
                SrcRect.top = 0;
                SrcRect.bottom = srcHeight;

                DstRect.left    = 0;
                DstRect.right   = dstWidth;
                DstRect.top     = 0;
                DstRect.bottom  = dstHeight;

                pVideoOpt->cap_handle = CSVID_OpenCapture(pVideoOpt->dev_index, srcWidth, srcHeight, input_interlace);
                if(NULL == pVideoOpt->cap_handle)
                {
                    printf("Open capture %d failed\n", pVideoOpt->dev_index);
                    return -1;
                }

                CSVID_SetSrcPosition(pVideoOpt->cap_handle,srcWidth, srcHeight, &SrcRect, input_interlace);
                CSVID_SetDstPosition(pVideoOpt->cap_handle,dstWidth, dstHeight, &DstRect, output_interlace);

                pVideoOpt->play_status = OPENED;
            }
        }
        else if(pVideoOpt->device_type == VID_DEV_TYPE_MULTI_CHANNEL)
        {
            if(pVideoOpt->play_status == CLOSED)
            {
                //CSHDMIRX_Open();
                //CSHDMIRX_SetManSigPhase(pVideoOpt->phase);
                //CSTVOUT_HDMI_IN_ENABLE(pVideoOpt->tvout_handle, 1);

                GetRxTvMode(&srcWidth, &srcHeight, &input_interlace);
                GetOutputTvMOde(pVideoOpt->tv_mode,&dstWidth, &dstHeight, &output_interlace);

                if(NULL == (pVideoOpt->dec_handle = CSVID_Open(pVideoOpt->dev_index)))
                {
                    printf("Open deooder %d failed\n", pVideoOpt->dev_index);
                    return -1;
                }

                pVideoOpt->cap_handle = CSVID_OpenCapture(pVideoOpt->dev_index, srcWidth, srcHeight, input_interlace);
                if(NULL == pVideoOpt->cap_handle)
                {
                    printf("Open capture %d failed\n", pVideoOpt->dev_index);
                    return -1;
                }

                memset(&EncAttr, 0, sizeof(EncCpbAttr));

                EncAttr.CodecStd = pVideoOpt->input_codec;
                EncAttr.SubFrameSyncEn = 0;
                EncAttr.PicFormat.Format3D=0;//FORMAT_2D;
                EncAttr.PicFormat.IsInterlace= input_interlace;
                EncAttr.PicFormat.HorSample=1;
                EncAttr.PicFormat.VerSample=1;
                EncAttr.PicFormat.TopFieldFirst=1;
                EncAttr.PicFormat.VactSpace1=0;
                EncAttr.PicFormat.VactSpace2=0;
                EncAttr.PicFormat.PicWidth= srcWidth;
                EncAttr.PicFormat.PicHeight= srcHeight;
                EncAttr.PicFormat.FrameRate= VID_FRAME_RATE_3000;

                EncAttr.EncOpenParam.FrameRateInfo = VID_FRAME_RATE(30);
                EncAttr.EncOpenParam.RateCtrlMode = 1;
                EncAttr.EncOpenParam.BitRate = 5000;
                EncAttr.EncOpenParam.IntraRefresh = 0;
                EncAttr.EncOpenParam.QpY = 20;
                EncAttr.EncOpenParam.EnableAutoSkip = 0;
                EncAttr.EncOpenParam.GopSize = 64;
                EncAttr.EncOpenParam.MeSearchRangeX = 0;
                EncAttr.EncOpenParam.MeSearchRangeY = 2;
                EncAttr.EncOpenParam.Profile = 2;

                pVideoOpt->enc_handle = CSVID_OpenEncoder(pVideoOpt->dev_index, &EncAttr);
                if(NULL == pVideoOpt->enc_handle)
                {
                    printf("Open encoder %d failed\n", pVideoOpt->dev_index);
                    return -1;
                }

                CSVID_SetStreamType(pVideoOpt->dec_handle, pVideoOpt->input_codec);
                CSVID_SetDecoderReorder(pVideoOpt->dec_handle, 1);
                CSVID_EnablePTSSync(pVideoOpt->dec_handle);
                CSVID_SetVideoWithAudio(pVideoOpt->dec_handle, 0);
                
                CSVID_SetLowLatencyMode(pVideoOpt->dec_handle, VID_LLD_STREAM_LOW_DELAY_MODE, 10);
                
                //CSVID_SyncNotify(pVideoOpt->dec_handle, synccall, 1000000, 1);

                SrcRect.left = 0;
                SrcRect.right = srcWidth;
                SrcRect.top = 0;
                SrcRect.bottom = srcHeight;

                DstRect.left    = 0;
                DstRect.right   = 960;
                DstRect.top     = 0;
                DstRect.bottom  = 600;

                CSVID_SetSrcPosition(pVideoOpt->cap_handle, srcWidth, srcHeight, &SrcRect, input_interlace);
                CSVID_SetDstPosition(pVideoOpt->cap_handle, dstWidth, dstHeight, &DstRect, output_interlace);

                CSVID_SetSrcPosition(pVideoOpt->enc_handle, srcWidth, srcHeight, &SrcRect, input_interlace);
                CSVID_SetDstPosition(pVideoOpt->enc_handle, srcWidth, srcHeight, &SrcRect, output_interlace);

                DstRect.left    = 960;
                DstRect.right   = 1920;
                DstRect.top     = 0;
                DstRect.bottom  = 600;

                CSVID_SetSrcPosition(pVideoOpt->dec_handle, srcWidth, srcHeight, &SrcRect, input_interlace);
                CSVID_SetDstPosition(pVideoOpt->dec_handle, dstWidth, dstHeight, &DstRect, output_interlace);

                pVideoOpt->play_status = OPENED;

                sem_init(&pVideoOpt->lock, 0, 0);

                if(0 != pthread_create(&pVideoOpt->task, NULL, do_from_encoder_to_decoder, pVideoOpt))
                {
                    printf("%d %s Create task failed\n", __LINE__, __FUNCTION__);
                }
            }
        }
        break;
    case 'c':
        if(pVideoOpt->device_type == VID_DEV_TYPE_DECODER)
        {
            if((pVideoOpt->play_status == OPENED) || (pVideoOpt->play_status == PLAY) || (pVideoOpt->play_status == STOP))
            {
                if(pVideoOpt->play_status == PLAY)
                {
                    sem_wait(&pVideoOpt->lock);
                    CSVID_Stop(pVideoOpt->dec_handle);
                }
                sem_destroy(&pVideoOpt->lock);
                if(pVideoOpt->input_fp)
                {
                    fclose(pVideoOpt->input_fp);
                    pVideoOpt->input_fp = NULL;
                }
                CSVID_Close(pVideoOpt->dec_handle);
                pVideoOpt->play_status = CLOSED;
                pVideoOpt->dec_handle = NULL;
            }
        }
        else if(pVideoOpt->device_type == VID_DEV_TYPE_ENCODER)
        {
            if((pVideoOpt->play_status == OPENED) || (pVideoOpt->play_status == PLAY) || (pVideoOpt->play_status == STOP))
            {
                if(pVideoOpt->play_status == PLAY)
                {
                    sem_wait(&pVideoOpt->lock);
                    CSVID_StopEncoder(pVideoOpt->enc_handle);
                }
                sem_destroy(&pVideoOpt->lock);
                if(pVideoOpt->input_fp)
                {
                    fclose(pVideoOpt->input_fp);
                    pVideoOpt->input_fp = NULL;
                }
                CSVID_CloseEncoder(pVideoOpt->enc_handle);
                pVideoOpt->play_status = CLOSED;
                pVideoOpt->enc_handle = NULL;
            }

        }
        else if(pVideoOpt->device_type == VID_DEV_TYPE_CAPTURE)
        {
            if((pVideoOpt->play_status == OPENED) || (pVideoOpt->play_status == PLAY) || (pVideoOpt->play_status == STOP))
            {
                if(pVideoOpt->play_status == PLAY)
                {
                    CSVID_StopCapture(pVideoOpt->cap_handle);
                }

                CSVID_CloseCapture(pVideoOpt->cap_handle);
                pVideoOpt->play_status = CLOSED;
                pVideoOpt->cap_handle = NULL;
            }
        }
        else if(pVideoOpt->device_type == VID_DEV_TYPE_MULTI_CHANNEL)
        {

            if((pVideoOpt->play_status == OPENED) || (pVideoOpt->play_status == PLAY) || (pVideoOpt->play_status == STOP))
            {
                if(pVideoOpt->play_status == PLAY)
                {
                    sem_wait(&pVideoOpt->lock);
                    CSVID_StopEncoder(pVideoOpt->enc_handle);
                    CSVID_Stop(pVideoOpt->dec_handle);
                    pVideoOpt->play_status = STOP;
                }
                sem_destroy(&pVideoOpt->lock);
                CSVID_Close(pVideoOpt->dec_handle);
                CSVID_CloseEncoder(pVideoOpt->enc_handle);
                CSVID_CloseCapture(pVideoOpt->cap_handle);

                pVideoOpt->dec_handle = NULL;
                pVideoOpt->enc_handle = NULL;
                pVideoOpt->cap_handle = NULL;
                pVideoOpt->play_status = CLOSED;
            }
        }
        break;
    case 'q':
        if(pVideoOpt->play_status == CLOSED)
        {
            CSVID_Term();
            return 1;
        }
        if(pVideoOpt->device_type == VID_DEV_TYPE_DECODER)
        {
            if(pVideoOpt->play_status == PLAY)
            {
                sem_wait(&pVideoOpt->lock);
                CSVID_Stop(pVideoOpt->dec_handle);
            }
            sem_destroy(&pVideoOpt->lock);
            if(pVideoOpt->input_fp)
            {
                fclose(pVideoOpt->input_fp);
                pVideoOpt->input_fp = NULL;
            }
            CSVID_SyncNotify(pVideoOpt->dec_handle, NULL, 1000000, 0);
            CSVID_Close(pVideoOpt->dec_handle);
        }
        else if(pVideoOpt->device_type == VID_DEV_TYPE_DECODER_SLOW)
        {
            if(pVideoOpt->play_status == PLAY)
            {
                sem_wait(&pVideoOpt->lock);
                CSVID_Stop(pVideoOpt->dec_handle);
            }
            sem_destroy(&pVideoOpt->lock);
            if(pVideoOpt->input_fp)
            {
                fclose(pVideoOpt->input_fp);
                pVideoOpt->input_fp = NULL;
            }
            CSVID_SyncNotify(pVideoOpt->dec_handle, NULL, 1000000, 0);
            CSVID_Close(pVideoOpt->dec_handle);
        }
        else if(pVideoOpt->device_type == VID_DEV_TYPE_ENCODER)
        {
            if(pVideoOpt->play_status == PLAY)
            {
                sem_wait(&pVideoOpt->lock);
                CSVID_StopEncoder(pVideoOpt->enc_handle);
            }
            sem_destroy(&pVideoOpt->lock);
            if(pVideoOpt->input_fp)
            {
                fclose(pVideoOpt->input_fp);
                pVideoOpt->input_fp = NULL;
            }
            CSVID_CloseEncoder(pVideoOpt->enc_handle);
        }
        else if(pVideoOpt->device_type == VID_DEV_TYPE_CAPTURE)
        {
            if(pVideoOpt->play_status == PLAY)
            {
                CSVID_StopCapture(pVideoOpt->cap_handle);
            }
            CSVID_CloseCapture(pVideoOpt->cap_handle);

        }
        else if(pVideoOpt->device_type == VID_DEV_TYPE_MULTI_CHANNEL)
        {
            if(pVideoOpt->play_status == PLAY)
            {
                sem_wait(&pVideoOpt->lock);
                CSVID_StopEncoder(pVideoOpt->enc_handle);
                CSVID_Stop(pVideoOpt->dec_handle);
                pVideoOpt->play_status = STOP;
            }
            sem_destroy(&pVideoOpt->lock);
            CSVID_Close(pVideoOpt->dec_handle);
            CSVID_CloseEncoder(pVideoOpt->enc_handle);
            CSVID_CloseCapture(pVideoOpt->cap_handle);
        }
        else if(pVideoOpt->device_type == VID_DEV_TYPE_TRANSCODING)
        {
            if(pVideoOpt->play_status == PLAY)
            {
                sem_wait(&pVideoOpt->lock);
                CSVID_StopTranscoding(pVideoOpt->dec_handle, pVideoOpt->enc_handle);
                pVideoOpt->play_status = STOP;
            }
            sem_destroy(&pVideoOpt->lock);
            CSVID_CloseTranscoding(pVideoOpt->dec_handle, pVideoOpt->enc_handle);
        }

        CSVID_Term();
        return 1;
    case 'y':
    {
        int val = 0;
        scanf("%d", &val);
        if(0 == val)
            CSVID_EnableYUVEvent(pVideoOpt->dec_handle, 0);
        else
            CSVID_EnableYUVEvent(pVideoOpt->dec_handle, 1);

        break;
    }
    case 'z':
        break;
    case 'e':
        CSVID_EnableOutput(pVideoOpt->dec_handle);
        printf("enable output\n");

        break;
    case 'd':
        CSVID_DisableOutput(pVideoOpt->dec_handle);
        printf("disable output\n");

        break;
    case 'g':
        src.CropYOff=20;
        src.CropYHeight=240;
        src.CropXOff=20;
        src.CropXWidth=320;
        dst.top=20+step*20;
        dst.bottom=240+step*20;
        dst.left=20+step*20;
        dst.right=320+step*20;
        step++;
        CSVID_SetOutputPostion(pVideoOpt->dec_handle, src, dst);
        printf("set pos\n");

        break;

    default:
        ;
    }
    return 0;
}

static int parse_video_cmdline(int argc, char **argv, VideoOpt_t *pVideoOpt)
{
    int ret = 0;
    int i = 1;

    while(i<argc)
    {

        if((0 == strcmp(argv[i], "-D") || (0 == strcmp(argv[i], "-d"))))
        {
            i += 1;
            if(0 == strcmp(argv[i], "DEC0"))
            {
                pVideoOpt->dev_index = VID_DEV_DEC0;
                pVideoOpt->device_type = VID_DEV_TYPE_DECODER;
            }
            else if(0 == strcmp(argv[i], "DEC0_STEP"))
            {
                pVideoOpt->dev_index = VID_DEV_DEC0;
                pVideoOpt->device_type = VID_DEV_TYPE_DECODER_SLOW;
            }
            else if(0 == strcmp(argv[i], "DEC1"))
            {
                pVideoOpt->dev_index = VID_DEV_DEC1;
                pVideoOpt->device_type = VID_DEV_TYPE_DECODER;
            }
            else if(0 == strcmp(argv[i], "ENC0"))
            {
                pVideoOpt->dev_index = VID_DEV_ENC0;
                pVideoOpt->device_type = VID_DEV_TYPE_ENCODER;
            }
            else if(0 == strcmp(argv[i], "ENC1"))
            {
                pVideoOpt->dev_index = VID_DEV_ENC1;
                pVideoOpt->device_type = VID_DEV_TYPE_ENCODER;
            }
            else if(0 == strcmp(argv[i], "CAP"))
            {
                pVideoOpt->dev_index = VID_DEV_CAP;
                pVideoOpt->device_type = VID_DEV_TYPE_CAPTURE;
            }
            else if(0 == strcmp(argv[i], "MULTI"))
            {
                pVideoOpt->dev_index = 0;
                pVideoOpt->device_type = VID_DEV_TYPE_MULTI_CHANNEL;
            }
            else if(0 == strcmp(argv[i], "TRANSCODING"))
            {
                pVideoOpt->dev_index = VID_DEV_ENC0;
                pVideoOpt->device_type = VID_DEV_TYPE_TRANSCODING;
            }
            else
            {
                printf("Incorrect device type\n");
                ret = -1;
                break;
            }
            i += 1;
        }
        else if((0 == strcmp(argv[i], "-P")) || (0 == strcmp(argv[i], "-p")))
        {
            i += 1;
            pVideoOpt->phase = atoi(argv[i]);
            i += 1;
        }
        else if((0 == strcmp(argv[i], "-I")) || (0 == strcmp(argv[i], "-i")))
        {
            i += 1;
            pVideoOpt->input_file = argv[i];
            i += 1;
        }
        else if((0 == strcmp(argv[i], "-O")) || (0 == strcmp(argv[i], "-o")))
        {
            i += 1;
            pVideoOpt->output_file = argv[i];
            i += 1;
        }
        else if((0 == strcmp(argv[i], "-R")) || (0 == strcmp(argv[i], "-r")))
        {
            i += 1;
            if(0 == strcmp(argv[i], "NTSC"))
            {
                pVideoOpt->tv_mode = TVOUT_MODE_480I;
            }
            else if(0 == strcmp(argv[i], "PAL"))
            {
                pVideoOpt->tv_mode = TVOUT_MODE_576I;
            }
            else if(0 == strcmp(argv[i], "720P50"))
            {
                pVideoOpt->tv_mode = TVOUT_MODE_720P50;
            }
            else if(0 == strcmp(argv[i], "720P60"))
            {
                pVideoOpt->tv_mode = TVOUT_MODE_720P60;
            }
            else if(0 == strcmp(argv[i], "1080I30"))
            {
                pVideoOpt->tv_mode = TVOUT_MODE_1080I30;
            }
            else if(0 == strcmp(argv[i], "1080P30"))
            {
                pVideoOpt->tv_mode = TVOUT_MODE_1080P30;
            }
            else if(0 == strcmp(argv[i], "1080P60"))
            {
                pVideoOpt->tv_mode = TVOUT_MODE_1080P60;
            }
            else
            {
                printf("Not support resolution\n");
                ret = -1;
                break;
            }
            i += 1;
        }
        else if((0 == strcmp(argv[i], "-C")) || (0 == strcmp(argv[i], "-c")))
        {
            i += 1;
            if(0 == strcmp(argv[i], "H264"))
            {
                pVideoOpt->input_codec = VID_STREAM_TYPE_H264;
            }
            else if(0 == strcmp(argv[i], "VC1"))
            {
                pVideoOpt->input_codec = VID_STREAM_TYPE_VC1;
            }
            else if(0 == strcmp(argv[i], "MPEG2"))
            {
                pVideoOpt->input_codec = VID_STREAM_TYPE_MPEG2;
            }
            else if(0 == strcmp(argv[i], "MPEG4"))
            {
                pVideoOpt->input_codec = VID_STREAM_TYPE_MPEG4;
            }
            else if(0 == strcmp(argv[i], "H263"))
            {
                pVideoOpt->input_codec = VID_STREAM_TYPE_H263;
            }
            else if(0 == strcmp(argv[i], "DIV3"))
            {
                pVideoOpt->input_codec = VID_STREAM_TYPE_DIVX;
            }
            else if(0 == strcmp(argv[i], "RV"))
            {
                pVideoOpt->input_codec = VID_STREAM_TYPE_RV;
            }
            else if(0 == strcmp(argv[i], "AVS"))
            {
                pVideoOpt->input_codec = VID_STREAM_TYPE_AVS;
            }
            else if(0 == strcmp(argv[i], "MJPG"))
            {
                pVideoOpt->input_codec = VID_STREAM_TYPE_MJPG;
            }
            else if(0 == strcmp(argv[i], "THO"))
            {
                pVideoOpt->input_codec = VID_STREAM_TYPE_THO;
            }
            else if(0 == strcmp(argv[i], "VP3"))
            {
                pVideoOpt->input_codec = VID_STREAM_TYPE_VP3;
            }
            else if(0 == strcmp(argv[i], "VP8"))
            {
                pVideoOpt->input_codec = VID_STREAM_TYPE_VP8;
            }
            else if(0 == strcmp(argv[i], "MVC"))
            {
                pVideoOpt->input_codec = VID_STREAM_TYPE_MVC;
            }
            else if(0 == strcmp(argv[i], "YUV"))
            {
                pVideoOpt->input_codec = VID_STREAM_TYPE_YUV;
            }
            else
            {
                printf("Not support codec type\n");
                ret = -1;
                break;
            }
            i += 1;
        }
        else if(0 == strcmp(argv[i], "-W"))
        {
            i += 1;
            pVideoOpt->width = atoi(argv[i]);
            i += 1;
        }
        else if(0 == strcmp(argv[i], "-H"))
        {
            i += 1;
            pVideoOpt->height = atoi(argv[i]);
            i += 1;
        }
    }

    return ret;
}
static void GetOutputTvMOde(CSTVOUT_MODE cur_mode, unsigned int *dstWidth, unsigned int *dstHeight, unsigned int *is_interlace)
{
    switch(cur_mode)
    {
    case TVOUT_MODE_576I:
    case TVOUT_MODE_PAL_N:
    case TVOUT_MODE_PAL_CN:
    case TVOUT_MODE_SECAM:
    case TVOUT_MODE_480I:
    case TVOUT_MODE_PAL_M:
    case TVOUT_MODE_1080I25:
    case TVOUT_MODE_1080I30:
        *is_interlace = 1;
        break;
    case TVOUT_MODE_576P:
    case TVOUT_MODE_480P:
    case TVOUT_MODE_720P50:
    case TVOUT_MODE_720P60:
    case TVOUT_MODE_1080P24:
    case TVOUT_MODE_1080P25:
    case TVOUT_MODE_1080P30:
    case TVOUT_MODE_1080P50:
    case TVOUT_MODE_1080P60:
    case TVOUT_RGB_640X480_60FPS:
    case TVOUT_RGB_800X600_60FPS:
    case TVOUT_RGB_1280X1024_60FPS:
    default:
        *is_interlace = 0;
    }

    switch(cur_mode)
    {
    case TVOUT_MODE_576I:
    case TVOUT_MODE_576P:
    case TVOUT_MODE_PAL_N:
    case TVOUT_MODE_PAL_CN:
    case TVOUT_MODE_SECAM:
        *dstWidth = 720;
        *dstHeight = 576;
        break;
    case TVOUT_MODE_480I:
    case TVOUT_MODE_480P:
    case TVOUT_MODE_PAL_M:
        *dstWidth = 720;
        *dstHeight = 480;
        break;
    case TVOUT_MODE_720P50:
    case TVOUT_MODE_720P60:
        *dstWidth = 1280;
        *dstHeight = 720;
        break;
    case TVOUT_MODE_1080I25:
    case TVOUT_MODE_1080I30:
    case TVOUT_MODE_1080P24:
    case TVOUT_MODE_1080P25:
    case TVOUT_MODE_1080P30:
    case TVOUT_MODE_1080P50:
    case TVOUT_MODE_1080P60:
        *dstWidth = 1920;
        *dstHeight = 1080;
        break;
    case TVOUT_RGB_640X480_60FPS:
        *dstWidth = 640;
        *dstHeight = 480;
        break;
    case TVOUT_RGB_800X600_60FPS:
        *dstWidth = 800;
        *dstHeight = 600;
        break;
    case TVOUT_RGB_1280X1024_60FPS:
        *dstWidth = 1024;
        *dstHeight = 1280;
        break;
    default:
        *dstWidth = 1280;
        *dstHeight = 720;
        break;
    }
}
static void GetRxTvMode(unsigned int *srcWidth, unsigned int *srcHeight, unsigned int *is_interlace)
{
    DF_VIDEO_FORMAT VIDEO_FORMAT  = 0;
    unsigned int fps, threeD;
    int aspect_ratio;

    CSHDMIRX_GetRxTvMode(&VIDEO_FORMAT, srcWidth, srcHeight, &fps, is_interlace, &aspect_ratio, &threeD);
}

